<#
.Description
    Script che verifica se l'account di amministratore locale in Windows 10 o Windows 11 è attivo, e in base a ciò, offre la possibilità di reimpostare o impostare la password dell'account.
.Author 
    Roberto Bonfa'
.Version
    1.1
#>

Import-Module -Name Microsoft.PowerShell.LocalAccounts

$adminAccount = Get-LocalUser -Name "Administrator"

function Get-ValidPasswordLength {
    do {
        $length = Read-Host "Inserisci la lunghezza desiderata per la password (8-32 characters)"
        $length = [int]$length
    } while (-not $length -or $length -lt 8 -or $length -gt 32)
    return $length
}

function Generate-Password {
    param ($length)
    [System.Web.Security.Membership]::GeneratePassword($length, 2)
}

function Set-AdminPassword {
    param ($adminAccount, $password)
    try {
        $securePassword = ConvertTo-SecureString $password -AsPlainText -Force
        $adminAccount | Set-LocalUser -Password $securePassword
        Write-Host "La nuova password è stata impostata"
    } catch {
        Write-Host "Errore nell'impostare la password: $_"
    }
}

function Save-PasswordToFile {
    param ($password)
    $saveLocation = Read-Host "Dove vuoi salvare il file contenete la nuova password? (Desktop/Documents)"
    if ($saveLocation -eq "Desktop") {
        $path = [Environment]::GetFolderPath("Desktop")
    } elseif ($saveLocation -eq "Documents") {
        $path = [Environment]::GetFolderPath("MyDocuments")
    } else {
        Write-Host "Percorso di salvataggio non valido. Il file verrà salvato in Documents."
        $path = [Environment]::GetFolderPath("MyDocuments")
    }
    $filePath = Join-Path $path "Administrator_Password.txt"
    $password | Out-File -FilePath $filePath
    Write-Host "Password salvata in '$filePath'"
}

if ($adminAccount.Enabled) {
    $resetPassword = Read-Host "L'account di Administrator locale è attivo. Vuoi resettare la password? (Y/N)"
    if ($resetPassword -eq "Y") {
        $passwordMethod = Read-Host "Vuoi inserire una password oppure generarne una automaticamente? (Enter/Generate)"
        if ($passwordMethod -eq "Enter") {
            $password = Read-Host "Scrivi la nuova password per l'account di Administrator locale" -AsSecureString
            $password = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($password))
            Set-AdminPassword -adminAccount $adminAccount -password $password
        } else {
            $passwordLength = Get-ValidPasswordLength
            $password = Generate-Password -length $passwordLength
            Set-AdminPassword -adminAccount $adminAccount -password $password
            Write-Host "Password generata: $password"
            $savePassword = Read-Host "Vuoi salvare la nuova password in un file di testo? (Y/N)"
            if ($savePassword -eq "Y") {
                Save-PasswordToFile -password $password
            }
        }
    }
} else {
    $activateAccount = Read-Host "L'account di Administrator locale NON è attivo. Vuoi attivarlo? (Y/N)"
    if ($activateAccount -eq "Y") {
        $passwordMethod = Read-Host "Vuoi inserire una password oppure generarne una automaticamente? (Enter/Generate)"
        if ($passwordMethod -eq "Enter") {
            $password = Read-Host "Scrivi la nuova password per l'account di Administrator locale" -AsSecureString
            $password = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($password))
            $adminAccount | Enable-LocalUser
            Set-AdminPassword -adminAccount $adminAccount -password $password
        } else {
            $passwordLength = Get-ValidPasswordLength
            $password = Generate-Password -length $passwordLength
            $adminAccount | Enable-LocalUser
            Set-AdminPassword -adminAccount $adminAccount -password $password
            Write-Host "Password generata: $password"
            $savePassword = Read-Host "Vuoi salvare la nuova password in un file di testo? (Y/N)"
            if ($savePassword -eq "Y") {
                Save-PasswordToFile -password $password
            }
        }
    }
}